/*

Designed by Shane Tolmie of KeyGhost corporation.  Freely distributable.
Questions and comments to shane@keyghost.com
Want to download 4Mb of Hi-Tech C FAQ and sample source code? http://www.keyghost.com/htpic
(also has tiny device to record keystrokes in hardware on PC)

Microprocessor: 	Microchip PIC16F876

Compiled with:		Hitech-C v7.84pl1, MPLAB v4.12

Note:							all references are to PIC16C7X PDF version of Microchip manual, DS30390E

Overall goal: 		led counter of student debt, 3 billion and +$32/sec, visible 50 metres away

*/

/*

TO DO:

DONE May have to use 2 ports each for strobe, oe bar and clk, as it might draw >20mA

DONE	Have <100% duty cycle by toggling oe bar, but, as total current/each segment is <20mA,
			and max is 8x200mA=1.6A, we're pretty safe. Its just brightness.

DONE	Lead zero blanking

DONE	Change freq

Change brightness

*/

//standard includes
#include	<pic.h>
#include	"always.h"
#include	"codes.h"
#include	"delay.h"

//see assignments for digit in main
bank3 unsigned char digit[24];

//see assignments for add in main
bank3 unsigned char add[24];

//converting digits to segments
//the msb of the segment is always 1 to give power to the commas on the display
const unsigned char segment[10]=
{
	0b11111110,	//0
	0b11000010,	//1
	0b10110111,	//2
	0b11010111,	//3
	0b11001011,	//4
	0b11011101,	//5
	0b11111101,	//6
	0b11000110,	//7
	0b11111111,	//8
	0b11001111,	//9
};

//program files
#include	"delay.c"
#include	"error.c"
#include	"convert.c"
#include	"serial.c"
#include	"ringbuff.c"
#include	"interrup.c"
#include	"math.c"

//debugging
//unsigned char db;

main()
{
	signed char i;
	unsigned int timeout_int;
	unsigned char tempport;

	//ADCON1=

	digit[23]=0;
	digit[22]=0;
	digit[21]=0;
	digit[20]=0;
	digit[19]=0;
	digit[18]=0;
	digit[17]=3;
//billion
	digit[16]=0;
	digit[15]=0;
	digit[14]=0;
//million
	digit[13]=0;
	digit[12]=0;
	digit[11]=0;
//,
	digit[10]=0;
	digit[9]=0;
	digit[8]=0;
//these bottom 8 digits are to reduce rounding errors
	digit[7]=0;
	digit[6]=0;
	digit[5]=0;
	digit[4]=0;
	digit[3]=0;
	digit[2]=0;
	digit[1]=0;
	digit[0]=0;

//what to add to each digit 4 times/sec
//bottom 8 digits are for rounding, and should normally
//	be 0 for incrementing in whole numbers
//to add $4.0000/sec, make add=100000000
//to add $3.9999/sec, make add=099997500
	add[23]=0;
	add[22]=0;
	add[21]=0;
	add[20]=0;
	add[19]=0;
	add[18]=0;
	add[17]=0;
//billion
	add[16]=0;
	add[15]=0;
	add[14]=0;
//million
	add[13]=0;
	add[12]=0;
	add[11]=0;
//thousand
	add[10]=0;
	add[9]=0;
	add[8]=1;
//these digits are to reduce rounding errors
	add[7]=0;
	add[6]=0;
	add[5]=0;
	add[4]=0;
	add[3]=0;
	add[2]=0;
	add[1]=0;
	add[0]=0;

/*
	//a very simple example of how this thing works
	//	minus the interrupts
	gie_off;
	while(1)
	{
		b;
		update_debt();
		b;
		digit_shift();
		b;
		DelayMs(250);
		b;
	}
*/

	
	setup_digit_shift_ports();
	setup_interrupts();
	setup_serial();

	//while (1) putch(getch()+1);

	//turn on display
	oe_bar_lo;

	//multi-tasking interrupt kernel
	//see interrupt routine for other portion
	while(1)
	{
		if (task2_go)
		{
			task2_go=FALSE;
			//update digits every 250ms		
			update_debt();
			digit_shift();
		}

		/*
		if (task3_go)
		{
			task3_go=FALSE;
			//every 1000ms fill up serial_out ring buffer with current debt digit[] and add[] info
			//do this in main so serial checking/generating is preserved
			serial_out_put('D');
			for (i=23;i>=0;i--)
			{
				serial_out_put(digit[i]+'0');
			}
			serial_out_put('A');
			for (i=23;i>=0;i--)
			{
				serial_out_put(add[i]+'0');
			}
		}
		*/

		if (task4_go)
		{
			task4_go=FALSE;
			//every 1100ms check the input serial ring buffer for 'D' or 'A', meaning get
			//	update of current debt[] and add[] information
			//	data is input 'D300000etc', ie: D followed by msb digit first, full 24 digits
			//do this in main so serial checking/generating is preserved

			while(serial_in_full)
			{
				//scan through buffer until commands 'A' or 'D' or 'X' are found
				switch (serial_in_get())
				{
					//give response code to say 'I'm here!'
					case 'X':
					{
						serial_out_put('Y');
						b;
						break;
					}

					//get new value to add to debt
					case 'A':
					{
						timeout_int=timeout_int_us(240100);
						while ( (timeout_int-->2) && (serial_in_count() < 24) )
						{
							//wait for 24 unsigned char digits to arrive
							//@9600bps should take less than 25ms
						}

						if (timeout_int>4)
						//all 24 digits arrived AOK
						{
							for (i=23;i>=0;i--)
							{
								add[i]=serial_in_get()-'0';	//digits come in as ascii, convert to bcd
							}
						}
						b;
						break;
					}

					//get new value of debt
					case 'D':
					{
						timeout_int=timeout_int_us(240100);
						while ( (timeout_int-->2) && (serial_in_count() < 24) )
						{
							//wait for 24 unsigned char digits to arrive
							//@9600bps should take less than 25ms
						}

						if (timeout_int>4)
						//all 24 digits arrived AOK
						{
							for (i=23;i>=0;i--)
							{
								digit[i]=serial_in_get()-'0';	//digits come in as ascii, convert to bcd
							}
						}
						b;
						break;
					}
					
					//get new value of update frequency
					case 'F':
					{
						timeout_int=timeout_int_us(240100);
						while ( (timeout_int-->2) && (serial_in_count() < 24) )
						{
							//wait for 24 unsigned char digits to arrive
							//@9600bps should take less than 25ms
						}

						if (timeout_int>4)
						//all 24 digits arrived AOK
						{
							task2_counter_max=0;
							for (i=23;i>=0;i--)
							{
								//task2 controls the speed of update of display,
								//	in 0.5ms increments, 500 represents 250ms
								//	max received value is 9999, min is 50
								//	get 24 digits to be compatible with other methods
								if (i<=3)
								{
									task2_counter_max+=(((unsigned int)serial_in_get())-'0') * uint_powerof(10,i);
								}
								else
								{
									//throw away digits that are zero for compatibility
									serial_in_get();
								}
							}
						}
						break;
					}					
				} //switch serial_in_get()
			} //while (serial_in_full)
		} //if (task4_go)

	} //while(1)
}

